//===----------- ThreeAddr2TwoAddr_Fer.cpp - Printing prototype -----------===//
//
//             Register Allocation via Coloring of Chordal Graphs 
//
// This pass converts a three address instruction to a two address instruction.
//
// Date: May 28th, 2006
//
//===----------------------------------------------------------------------===//
//
// In LLVM, all the instructions are three address code, but some target
// architectures have two address instructions. Therefore, it is necessary to
// make this conversion, what is exaclty what this class does.
//
//===----------------------------------------------------------------------===//

#ifndef THREE_ADDR_2_TWO_ADDR_FER
#define THREE_ADDR_2_TWO_ADDR_FER

using namespace llvm;

namespace llvm {

    class ThreeAddr2TwoAddr_Fer {

        public:
 
            // Checkes each instruction in the function, and converts the three
            // address instructions to two address if necessary.
            bool runOnMachineFunction(MachineFunction & mf);

        private:
 
            // The function that is been compiled.
            MachineFunction * mf;

            // Information about the set of available registers.
            const MRegisterInfo * ri;

            // Information about the instructions. Notice that the current
            // implementation of this class is very limited in the sense that
            // it expects a very particular type of three address instruction.
            const TargetInstrInfo * tii;

            // Finds the two address instructions in the basic block and
            // converts them to two address instructions. We assume that, if
            // the instruction is in the format a := b + a, then it is
            // commutative. We also assume that the first three operands of the
            // instruction are registers.
            bool remove3address(MachineBasicBlock & mbb);
    };

}

#endif
